<?php

/**
 * Cough Web Panel - A quick, simple, interface to the CoughGenerator
 * class.
 * 
 * @author Anthony Bush
 * @version $Id$
 * @copyright Anthony Bush (http://anthonybush.com/), 30 April, 2007
 * @package default
 **/

// Get the cough generator classes
include('cough_generator_classes.php');

// Setup some web panel contants
define('CONFIG_FILE_SUFFIX', '.php');
define('CONFIG_PATH', dirname(__FILE__) . '/config/');

// Load the configs
$configs = array();
$configDir = dir(CONFIG_PATH);
while ($file = $configDir->read()) {
	if (strpos($file, '.') !== 0 && strstr($file, CONFIG_FILE_SUFFIX) == CONFIG_FILE_SUFFIX) {
		include_once(CONFIG_PATH . $file);
		if (isset($config)) {
			$configs[$file] = $config;
			unset($config);
		}
	}
}

echo '<pre>';
print_r($configs);
echo '</pre>';
die();

		##############
		# Controller #
		##############

		$flashMessage = '';
		
		// Action: Generate Cough Classes
		if (isset($_GET['generate'])) {
			$databaseID = intval($_GET['generate']);
			$generation = new CoughGeneration($databaseID);
			$dbName      = $generation->getDatabaseName();
			$classPrefix = $generation->getClassPrefix();
			$generator = new AcademicSuperstoreGenerator();
			if ($generator->generate($dbName, $classPrefix)) {
				$flashMessage = 'Success generating ' . $dbName . " classes.<br />\n";
				$generation->setGenerationDate(date('Y-m-d H:i:s'));
				$generation->setGeneratedBy($user);
				$generation->save();
			} else {
				$flashMessage  = 'Failure generating ' . $dbName . " classes<br />\n";
				$errorMessages = $generator->getErrorMessages();
				if ( ! empty($errorMessages)) {
					$flashMessage .= 'Error(s) reported by CoughGenerator:' . "<br />\n";
					$flashMessage .= implode('<br />', $errorMessages) . "<br />\n";
				}
			}
			$warnings = $generator->getWarnings();
			$removedFiles = $generator->getRemovedFiles();
			$modifiedFiles = $generator->getModifiedFiles();
			$addedFiles = $generator->getAddedFiles();
		}


		########
		# View #
		########

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
	<title>Cough Web Panel</title>
	<style type="text/css" media="screen">
	/* <![CDATA[ */
		
		/* Generations Table */
		.genenerations {
			border-top: 1px solid #999;
			border-left: 1px solid #999;
		}
		.genenerations th {
			background: #ccc;
			padding: 3px;
			border-right: 1px solid #999;
			border-bottom: 1px solid #999;
		}
		.genenerations td {
			padding: 3px;
			border-right: 1px solid #999;
			border-bottom: 1px solid #999;
		}
		
		/* Results */
		.results {
			border: 1px solid #999;
			margin: 1em 0;
			background: #ccc;
		}
		.result_message {
			background: #FFFF99;
			padding: 5px;
			border-bottom: 1px solid #999;
		}
		.message {
			border: 1px solid #999;
			background: #eee;
			margin: 5px;
		}
		.message .title {
			padding: 3px 5px;
		}
		.message .content {
			font-family: monospace;
			background: #fff;
			padding: 5px;
			white-space: nowrap;
			overflow: auto;
		}
	/* ]]> */
	</style>
</head>
<body>

<h1>Cough Web Panel</h1>

<p>This page contains a list of the databases for which we are currently letting CoughGenerator generate code for. To (re)generate code for a database and all its tables, click the 'Regenerate' link for that database. It will generate the results to the 'shared/models/generated/' folder on the '<?=$user?>' fork, at which point <?=$user?> will need to commit the generated code for others to use it.</p>

<table border="0" cellspacing="0" cellpadding="0" class="genenerations">
	<tr>
		<th>Database</th>
		<th>Class Prefix</th>
		<th>Last generated on</th>
		<th>Last generated by</th>
		<th>Controls</th>
	</tr>
<?php foreach ($generations as $generationID => $generation): ?>
	<tr>
		<td><?=$generation->getDatabaseName()?></td>
		<td><?=$generation->getClassPrefix()?></td>
		<td><?=$generation->getGenerationDate()?></td>
		<td><?=$generation->getGeneratedBy()?></td>
		<td><a href="?page=cough_web_panel&amp;generate=<?=$generation->getKeyID()?>">Regenerate</a></td>
	</tr>
<?php endforeach; ?>
</table>

<form action="<?=$_SERVER['REQUEST_URI']?>" method="post" style="margin-top: 1em;">
<input type="submit" value="Add a database to above:">: <input type="text" name="add_database" /> with class prefix <input type="text" name="class_prefix" />
</form>


<?php if ( ! empty($flashMessage)): ?>

<div class="results">

	<div class="result_message"><?=$flashMessage?></div>

	<div class="message">
		<div class="title">Warnings</div>
		<div class="content">
		
			<?php
			if (empty($warnings)) {
				echo 'No Warnings';
			} else {
				foreach ($warnings as $warning) {
					echo $warning . "<br />\n";
				}
			}
			?>
		
		</div>
	</div>
	
	<div class="message">
		<div class="title">Added Files</div>
		<div class="content">
		
			<?php
			if (empty($addedFiles)) {
				echo 'No Added Files';
			} else {
				foreach ($addedFiles as $filename) {
					echo $filename . "<br />\n";
				}
				echo "<br />\n";
				echo 'cvs add ' . str_replace(SHARED_PATH, '', implode(' ', $addedFiles)) . '';
			}
			?>
		
		</div>
	</div>
	
	<div class="message">
		<div class="title">Removed Files</div>
		<div class="content">
		
			<?php
			if (empty($removedFiles)) {
				echo 'No Removed Files';
			} else {
				foreach ($removedFiles as $filename) {
					echo $filename . "<br />\n";
				}
				echo "<br />\n";
				echo 'cvs remove ' . str_replace(SHARED_PATH, '', implode(' ', $removedFiles)) . '';
			}
			?>
		
		</div>
	</div>
	
	<div class="message">
		<div class="title">Modified Files</div>
		<div class="content">
		
			<?php
			if (empty($modifiedFiles)) {
				echo 'No Modified Files';
			} else {
				foreach ($modifiedFiles as $filename) {
					echo $filename . "<br />\n";
				}
			}
			?>
		
		</div>
	</div>
	
</div>

<?php endif; ?>


</body>
</html>
